# -*- coding: utf-8 -*-
"""
/***************************************************************************
 MarinrTools
 A QGIS plugin
 Marine tools for analysis and interpretation.  These tools include:
 
 Benthic Terrain Modeller - geomorphological interpretation of bathymetry data
 OBIA - Object Based Image Analysis using a simple k-means algorithm
 MBES Segmentation - Classification using OBIA on bathymetry and backscatter data from a Multibeam Echosounder
 Bathymetry Morphometry - Create several derivatives from bathymetry data
 Tools for multibeam survey design (new surveys and filling in surveys)
 Imagery manipulation - removing outliers and changing null settings
 
 Partially generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2024-3-4
        git sha              : $Format:%H$
        copyright            : (C) 2024 by Tim Le Bas
        email                : tlb@noc.ac.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt5.QtWidgets import QAction, QMenu
from PyQt5.QtCore import QCoreApplication, QObject
from qgis.PyQt.QtGui import QIcon
import marinetools
import os.path

class MarineTools:
    def __init__( self, iface ):
        self.iface = iface

    def initGui(self):
        plugin_dir = os.path.dirname(__file__)
        
        menuBar = self.iface.mainWindow().menuBar()
        surveyDesignMenu = QMenu( QCoreApplication.translate("MarineTools","MarineTools") ,  menuBar)

        self.actionRemoveValues = QAction( 'Alter Raster Values',  self.iface.mainWindow() )
        self.actionRemoveValues.setIcon(QIcon(plugin_dir+"/icons/RemoveValues.png"))
        self.actionRemoveValues.triggered.connect( self.RemoveValues1 )
        surveyDesignMenu.addAction( self.actionRemoveValues )
        
        self.actionBathyMorph = QAction( 'Bathymetry Morphometry',  self.iface.mainWindow() )
        self.actionBathyMorph.setIcon(QIcon(plugin_dir+"/icons/BathyMorph.png"))
        self.actionBathyMorph.triggered.connect( self.BathyMorph1 )
        surveyDesignMenu.addAction( self.actionBathyMorph )
        
        self.actionBTM = QAction( 'Benthic Terrain Modeller',  self.iface.mainWindow() )
        self.actionBTM.setIcon(QIcon(plugin_dir+"/icons/BTM.png"))
        self.actionBTM.triggered.connect( self.BTM1 ) # Link to def below
        surveyDesignMenu.addAction( self.actionBTM )

        self.actionEstimateCoverage = QAction( 'Estimate new MBES Coverage',  self.iface.mainWindow() )
        self.actionEstimateCoverage.setIcon(QIcon(plugin_dir+"/icons/estimatecoverage.png"))
        self.actionEstimateCoverage.triggered.connect( self.EstimateCoverage1 )
        surveyDesignMenu.addAction( self.actionEstimateCoverage )
        
        self.actionMBESseg = QAction( 'MBES segmentation',  self.iface.mainWindow() )
        self.actionMBESseg.setIcon(QIcon(plugin_dir+"/icons/MBESseg.png"))
        self.actionMBESseg.triggered.connect( self.MBESseg1 )
        surveyDesignMenu.addAction( self.actionMBESseg )
        
        self.actionOBIA = QAction( 'OBIA',  self.iface.mainWindow() )
        self.actionOBIA.setIcon(QIcon(plugin_dir+"/icons/OBIA.png"))
        self.actionOBIA.triggered.connect( self.OBIA1 )
        surveyDesignMenu.addAction( self.actionOBIA )
        
        self.actionRasterCoverage = QAction( 'Raster Coverage',  self.iface.mainWindow() )
        self.actionRasterCoverage.setIcon(QIcon(plugin_dir+"/icons/rastercoverage.png"))
        self.actionRasterCoverage.triggered.connect( self.RasterCoverage1 )
        surveyDesignMenu.addAction( self.actionRasterCoverage )
        
        self.actionSurveyLines = QAction( 'Survey Lines',  self.iface.mainWindow() )
        self.actionSurveyLines.setIcon(QIcon(plugin_dir+"/icons/surveylines.png"))
        self.actionSurveyLines.triggered.connect( self.SurveyLines1 )
        surveyDesignMenu.addAction( self.actionSurveyLines )
        
        self.surveyDesignAction = menuBar.addMenu( surveyDesignMenu )

    def unload(self):
        self.iface.mainWindow().menuBar().removeAction( self.surveyDesignAction )

    def BathyMorph1(self):
        from marinetools.bathymorph.BathyMorph_dialog import BathyMorphDialog
        from marinetools.bathymorph import BathyMorph
        self.first_start = True
        BathyMorph.BathyMorph.run(self)

    def BTM1(self):
        from marinetools.btm.BTM_dialog import BTMDialog
        from marinetools.btm import BTM
        self.first_start = True
        BTM.BTM.run(self)

    def EstimateCoverage1(self):
        from marinetools.estimatecoverage.EstimateCoverage_dialog import EstimateCoverageDialog
        from marinetools.estimatecoverage import EstimateCoverage
        self.first_start = True
        EstimateCoverage.EstimateCoverage.run(self)
        
    def MBESseg1(self):
        from marinetools.MBESseg.MBESseg_dialog import MBESsegDialog
        from marinetools.MBESseg import MBESseg
        self.first_start = True
        MBESseg.MBESseg.run(self)
        
    def OBIA1(self):
        from marinetools.obia.OBIA_dialog import OBIADialog
        from marinetools.obia import OBIA
        self.first_start = True
        OBIA.OBIA.run(self)

    def RasterCoverage1(self):
        from marinetools.rastercoverage.RasterCoverage_dialog import RasterCoverageDialog
        from marinetools.rastercoverage import RasterCoverage
        self.first_start = True
        RasterCoverage.RasterCoverage.run(self)

    def RemoveValues1(self):
        from marinetools.removevalues.RemoveValues_dialog import RemoveValuesDialog
        from marinetools.removevalues import RemoveValues
        self.first_start = True
        RemoveValues.RemoveValues.run(self)

    def SurveyLines1(self):
        from marinetools.surveylines.SurveyLines_dialog import SurveyLinesDialog
        from marinetools.surveylines import SurveyLines
        self.first_start = True
        SurveyLines.SurveyLines.run(self)

